#include "YumIO.h"
#include "YumIOCapturersMinion.h"
#include <QtCore/QThread>
#include <QtCore/QTimer>
#include <QtCore/QFile>

#define CHUNK_SIZE  8192

YumIOCapturersMinion::YumIOCapturersMinion(YumIO*  yumIO,
                                           int     stdoutReadFd,
                                           int     stderrReadFd)
   : yumIO(yumIO),
     initialized(false),
     stdoutReadFd(stdoutReadFd),
     stderrReadFd(stderrReadFd),
     stdoutReadNotifier(NULL),
     stderrReadNotifier(NULL),
     finalizeCompleted(1)
{
}

void  YumIOCapturersMinion::activate()
{
   // This is so that initialize will execute in the main thread
   QMetaObject::invokeMethod(this, "initialize", Qt::QueuedConnection);
}

void  YumIOCapturersMinion::initialize()
{
   if (!initialized) {
      stdoutReadNotifier = new QSocketNotifier(stdoutReadFd,
                                               QSocketNotifier::Read);
      stderrReadNotifier = new QSocketNotifier(stderrReadFd,
                                               QSocketNotifier::Read);

      connect(stdoutReadNotifier, SIGNAL(activated(int)),
              this,               SLOT(readStdout()));
      connect(stderrReadNotifier, SIGNAL(activated(int)),
              this,               SLOT(readStderr()));

      initialized = true;
   }
}

void  YumIOCapturersMinion::deactivate()
{
   // This is so that finalize will execute in the main thread
   finalizeCompleted.acquire();
   QMetaObject::invokeMethod(this, "finalize", Qt::QueuedConnection);
   // The finalize must happen syncronously so that finalize()'s calls to
   // readStdout() and readStderr() don't race with subsequent readStdout() or 
   // readStderr() calls (e.g. from ~YumIOCapturersMinion()).
   finalizeCompleted.acquire();
   finalizeCompleted.release();
}

void  YumIOCapturersMinion::finalize()
{
   if (initialized) {
      disconnect(stdoutReadNotifier, SIGNAL(activated(int)),
                 this,               SLOT(readStdout()));
      disconnect(stderrReadNotifier, SIGNAL(activated(int)),
                 this,               SLOT(readStderr()));

      readStdout();
      readStderr();

      initialized = false;
   }

   // Alert deactivate() that the finalize() is completed.
   finalizeCompleted.release();
}

YumIOCapturersMinion::~YumIOCapturersMinion()
{
   readStdout();
   readStderr();

   if (stdoutReadNotifier) delete stdoutReadNotifier;
   if (stderrReadNotifier) delete stderrReadNotifier;

   close(stdoutReadFd);
   close(stderrReadFd);
}

void  YumIOCapturersMinion::moveToThread(QThread*  targetThread)
{
   QObject::moveToThread(targetThread);
}

void  YumIOCapturersMinion::readStdout ()
{
   QFile  file;
   file.open(stdoutReadFd, QIODevice::ReadOnly);
   yumIO->writeStdout(file.readAll());
}

void  YumIOCapturersMinion::readStderr ()
{
   QFile  file;
   file.open(stderrReadFd, QIODevice::ReadOnly);
   yumIO->writeStderr(file.readAll());
}
